@@ -34,7 +34,8 @@
 
 class Animator : public GameHook {
  public:
-  Animator(Real length, Real position, Real direction, Real v0, Real v1, int repeat, SCM fun);
+  Animator(Game& g, Real length, Real position, Real direction, Real v0, Real v1, int repeat,
+           SCM fun);
   virtual ~Animator();
   virtual void tick(Real);
   virtual void releaseCallbacks();
