@@ -40,8 +40,6 @@
 #define BALL_NORMAL 1
 #define BALL_HIRES 2
 
-#define PHYSICS_RESOLUTION 0.002
-
 #include "animated.h"
 
 class Cell;
@@ -50,7 +48,7 @@ class Cell;
     (player/opponents/babies) are inherited */
 class Ball : public Animated {
  public:
-  explicit Ball(int role);
+  explicit Ball(Game& g, int role);
   virtual ~Ball();
   bool physics(Real time);
 
@@ -74,6 +72,7 @@ class Ball : public Animated {
   double nextJumpStrength;
   double rotoacc[2];
   double acceleration;
+  float dizzy_r, dizzy_p;
 
   /* radius is the *effective* radius, realRadius the real one */
   Real radius, realRadius;
