@@ -29,7 +29,7 @@
 
 class Bird : public Animated {
  public:
-  Bird(Real x, Real y, Real dx, Real dy, Real size, Real speed);
+  Bird(Game& g, Real x, Real y, Real dx, Real dy, Real size, Real speed);
 
   virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
   virtual void drawBuffers1(const GLuint*) const;
