@@ -27,10 +27,10 @@
 #include "map.h"
 #include "sound.h"
 
-Cactus::Cactus(Real x, Real y, Real radius) : Animated(Role_OtherAnimated, 1) {
+Cactus::Cactus(Game &g, Real x, Real y, Real radius) : Animated(g, Role_OtherAnimated, 1) {
   position[0] = x;
   position[1] = y;
-  position[2] = Game::current->map->getHeight(position[0], position[1]);
+  position[2] = game.map->getHeight(position[0], position[1]);
   primaryColor = Color(0.3, 1., 0.4, 1.);
   secondaryColor = Color(0., 0., 0., 1.);
   this->killed = false;
@@ -108,7 +108,6 @@ void Cactus::generateBuffers(const GLuint *idxbufs, const GLuint *databufs,
   GLfloat spike_rad[4] = {0.89f, 1.2f, 0.99f, 0.44f};
   for (int h = 0; h < 4; h++) {
     for (int i = 0; i < nsides; i++) {
-      GLfloat angle = 2 * i * M_PI / nsides + M_PI / nsides;
       GLfloat end[3] = {spike_rad[h] * radius * (GLfloat)cos12[2 * i + 1],
                         spike_rad[h] * radius * (GLfloat)sin12[2 * i + 1], spike_height[h]};
       GLfloat dhs[3][2] = {{0, spikewid}, {-spikewid, -spikewid}, {spikewid, -spikewid}};
@@ -166,7 +165,7 @@ void Cactus::drawBuffers2(const GLuint * /*idxbufs*/) const {}
 void Cactus::tick(Real t) {
   Animated::tick(t);
 
-  position[2] = Game::current->map->getHeight(position[0], position[1]);
+  position[2] = game.map->getHeight(position[0], position[1]);
 
   if (killed) {
     killed_time -= t;
@@ -176,7 +175,7 @@ void Cactus::tick(Real t) {
 
   // do I parse all the balls (incl. Mr Black) or just the player ?
   Animated **balls;
-  int nballs = Game::current->balls->bboxOverlapsWith(this, &balls);
+  int nballs = game.balls->bboxOverlapsWith(this, &balls);
   for (int i = 0; i < nballs; i++) {
     Ball *ball = (Ball *)balls[i];
     if (ball->no_physics) continue;
