@@ -26,7 +26,7 @@
 
 class Cactus : public Animated {
  public:
-  Cactus(Real x, Real y, Real radius);
+  Cactus(Game& g, Real x, Real y, Real radius);
 
   virtual void generateBuffers(const GLuint*, const GLuint*, const GLuint*, bool) const;
   virtual void drawBuffers1(const GLuint*) const;
