@@ -24,23 +24,13 @@
 
 #include "general.h"
 
-class Font {
- public:
-  static void init();
-
-  /* functions with a global access (not object-dependant)
-   * can be called anyway, after 'init'.
-   */
-  static void tick(Real t);
-
-  /* Return value is always the width of the rendered text */
-  static int drawSimpleText(const char *text, int x, int y, int sz, float r, float g, float b,
-                            float a, int maxwidth = 0);
-  static int drawRightSimpleText(const char *text, int x, int y, int sz, float r, float g,
-                                 float b, float a, int maxwidth = 0);
-  static int drawCenterSimpleText(const char *text, int x, int y, int sz, float r, float g,
-                                  float b, float a, int maxwidth = 0);
-  static int getTextWidth(const char *text, int sz, int maxwidth = 0);
-};
+/* Return value is always the width of the rendered text */
+int drawSimpleText(const char *text, int x, int y, int sz, float r, float g, float b, float a,
+                   int maxwidth = 0);
+int drawRightSimpleText(const char *text, int x, int y, int sz, float r, float g, float b,
+                        float a, int maxwidth = 0);
+int drawCenterSimpleText(const char *text, int x, int y, int sz, float r, float g, float b,
+                         float a, int maxwidth = 0);
+int getTextWidth(const char *text, int sz, int maxwidth = 0);
 
 #endif
