@@ -33,7 +33,7 @@
 #define mkdir(a, ...) mkdir(a)
 #endif
 
-extern char effectiveShareDir[];
+extern char effectiveShareDir[], effectiveLocalDir[], username[];
 
 /*** More includes follows after typedeclarations ***/
 
@@ -128,8 +128,6 @@ class Color {
 #endif
 
 void generalInit();
-double semiRand(int, int);
-double semiRand(int, int, int);
 double frandom();
 
 int fileExists(char *);
@@ -142,6 +140,7 @@ int pathIsLink(char *path); /** Checks if the given path points to a link */
 /* A modulus operations which handles negative results safely */
 int mymod(int v, int m);
 /* Return the current time, and calculate elapsed time in seconds */
+extern double timeDilationFactor;
 struct timespec getMonotonicTime();
 double getTimeDifference(const struct timespec &from, const struct timespec &to);
 
@@ -153,6 +152,6 @@ void warning(const char *formatstr, ...);
 extern int low_memory;  // if true, attempt to conserve memory
 extern int debug_joystick;
 extern int repair_joystick;
-extern int theFrameNumber;
+extern int displayFrameNumber;
 
 #endif /*GENERAL_H*/
