@@ -49,7 +49,6 @@ static GLuint shaderObjectDay = 0;
 static GLuint shaderObjectNight = 0;
 static GLuint shaderObjectShadow = 0;
 static GLuint shaderReflection = 0;
-static GLuint defaultVao = 0;
 GLuint textureBlank = 0;
 GLuint textureGlitter = 0;
 GLuint textureMousePointer = 0;
@@ -60,7 +59,6 @@ GLuint textureTrack = 0;
 TTF_Font *ingameFont;
 extern struct timespec displayStartTime;
 extern struct timespec lastDisplayStartTime;
-extern double timeDilationFactor;
 
 const GLfloat menuColorSelected[4] = {0.86f, 0.86f, 0.86f, 1.f};
 const GLfloat menuColor[4] = {0.86f, 0.86f, 0.25f, 1.f};
@@ -107,13 +105,6 @@ const double cos14[14] = {1.0,
                           0.6234898018587336,
                           0.9009688679024191};
 
-double fake_rand[4711];
-double frand(size_t i) {
-  /* size_t is nonnegative so no inputs will crash */
-  return fake_rand[i % 4711];
-}
-double frand() { return (rand() % (1 << 30)) / ((double)(1 << 30)); }
-
 #define GLHELP_MAX_TEXTURES 256
 GLuint textures[GLHELP_MAX_TEXTURES] = {0};  // added init. to 0 (no texture)
 char *textureNames[GLHELP_MAX_TEXTURES] = {NULL};
@@ -131,10 +122,10 @@ Sparkle2D *sparkle2D = NULL;
 struct StringInfo {
   TTF_Font *font;
   char string[256];
-  SDL_Color color;
 };
-int operator<(const struct StringInfo &a, const struct StringInfo &b) {
-  return memcmp(&a, &b, sizeof(struct StringInfo)) < 0;
+static bool operator<(const struct StringInfo &a, const struct StringInfo &b) {
+  if (a.font != b.font) return a.font < b.font;
+  return strcmp(a.string, b.string) < 0;
 }
 
 struct StringCache {
@@ -144,20 +135,22 @@ struct StringCache {
   long tick;
 };
 
-static long stringTick = 0;
 static std::map<StringInfo, StringCache> strcache;
 
 static SDL_Surface *drawStringToSurface(struct StringInfo &inf, bool outlined) {
+  SDL_Color white = {255, 255, 255, 255};
+  SDL_Color black = {0, 0, 0, 0};
   if (outlined) {
     TTF_SetFontOutline(inf.font, 0);
-    SDL_Surface *inner = TTF_RenderUTF8_Blended(inf.font, inf.string, inf.color);
+
+    SDL_Surface *inner = TTF_RenderUTF8_Blended(inf.font, inf.string, white);
     if (!inner) {
       warning("Failed to render string outline '%s'", inf.string);
       return NULL;
     }
     TTF_SetFontOutline(inf.font, 2);
-    SDL_Color blkColor = {0, 0, 0, 0};
-    SDL_Surface *outline = TTF_RenderUTF8_Blended(inf.font, inf.string, blkColor);
+
+    SDL_Surface *outline = TTF_RenderUTF8_Blended(inf.font, inf.string, black);
     if (!outline) {
       warning("Failed to render string inside '%s'", inf.string);
       return NULL;
@@ -168,7 +161,7 @@ static SDL_Surface *drawStringToSurface(struct StringInfo &inf, bool outlined) {
     SDL_FreeSurface(inner);
     return outline;
   } else {
-    SDL_Surface *outline = TTF_RenderUTF8_Blended(inf.font, inf.string, inf.color);
+    SDL_Surface *outline = TTF_RenderUTF8_Blended(inf.font, inf.string, white);
     if (!outline) {
       warning("Failed to render string '%s'", inf.string);
       return NULL;
@@ -180,17 +173,13 @@ static SDL_Surface *drawStringToSurface(struct StringInfo &inf, bool outlined) {
 int draw2DString(TTF_Font *font, const char *string, int x, int y, float red, float green,
                  float blue, float alpha, bool outlined, int align, int maxwidth) {
   struct StringInfo inf;
-  inf.color.r = 255 * red;
-  inf.color.g = 255 * green;
-  inf.color.b = 255 * blue;
-  inf.color.a = 255 * alpha;
   inf.font = font;
   memset(inf.string, 0, 256);
   strncpy(inf.string, string, 255);
   inf.string[255] = '\0';
   if (strcache.count(inf) <= 0) {
     struct StringCache newentry;
-    newentry.tick = stringTick;
+    newentry.tick = displayFrameNumber;
     SDL_Surface *surf = drawStringToSurface(inf, outlined);
     if (!surf) { return 0; }
     newentry.texture = LoadTexture(surf, newentry.texcoord);
@@ -201,23 +190,24 @@ int draw2DString(TTF_Font *font, const char *string, int x, int y, float red, fl
   }
 
   struct StringCache &cached = strcache[inf];
-  cached.tick = stringTick;
+  cached.tick = displayFrameNumber;
 
   GLfloat shrink = (maxwidth > 0 && maxwidth < cached.w) ? (maxwidth / (GLfloat)cached.w) : 1.;
   draw2DRectangle(x - shrink * align * cached.w / 2, y - shrink * cached.h / 2,
                   shrink * cached.w, shrink * cached.h, cached.texcoord[0], cached.texcoord[1],
-                  cached.texcoord[2], cached.texcoord[3], 1., 1., 1., 1., cached.texture);
+                  cached.texcoord[2], cached.texcoord[3], red, green, blue, alpha,
+                  cached.texture);
   return maxwidth > 0 ? std::min(cached.w, maxwidth) : cached.w;
 }
 
-void update2DStringCache() {
-  stringTick++;
+void update2DStringCache(bool force_wipe) {
   int erased;
   do {
     erased = false;
+    // TODO: better asymptotics
     for (std::map<StringInfo, StringCache>::iterator i = strcache.begin(); i != strcache.end();
          ++i) {
-      if (i->second.tick < stringTick - 100) {
+      if (i->second.tick < displayFrameNumber - 100 || force_wipe) {
         glDeleteTextures(1, &i->second.texture);
         strcache.erase(i);
         erased = true;
@@ -243,28 +233,32 @@ void tickMouse(Real td) {
   int mouseX, mouseY;
   static int oldMouseX = 0, oldMouseY = 0;
   SDL_GetMouseState(&mouseX, &mouseY);
-  double mouseSpeedX = (mouseX - oldMouseX) / td;
-  double mouseSpeedY = (mouseY - oldMouseY) / td;
+  float mouseSpeedX = (mouseX - oldMouseX) / td;
+  float mouseSpeedY = (mouseY - oldMouseY) / td;
+  float mouseSpeedLen = std::sqrt(mouseSpeedX * mouseSpeedX + mouseSpeedY * mouseSpeedY);
+  if (mouseSpeedLen > 25.f) {
+    mouseSpeedX /= 25.f / mouseSpeedLen;
+    mouseSpeedY /= 25.f / mouseSpeedLen;
+    mouseSpeedLen = 25.f;
+  }
   static Real last_sparkle = 0.0;
-  if (mouseSpeedX > 20.) mouseSpeedX = 20.;
-  if (mouseSpeedY > 20.) mouseSpeedY = 20.;
-  if (mouseSpeedX < -20.) mouseSpeedX = -20.;
-  if (mouseSpeedY < -20.) mouseSpeedY = -20.;
 
   oldMouseX = mouseX;
   oldMouseY = mouseY;
 
   mousePointerPhase += td;
   sparkle2D->tick(td);
-  last_sparkle += td * (1.0 + (fabs(mouseSpeedX) + fabs(mouseSpeedY)) * 0.1);
+  last_sparkle += td * (1.0 + mouseSpeedLen * 0.1);
   while (last_sparkle > 0.0) {
     last_sparkle -= 0.05;
-    sparkle2D->add(
-        (float)mouseX + 10.0 * (frandom() - 0.5), (float)mouseY + 10.0 * (frandom() - 0.5),
-        (float)mouseSpeedX / 2. + 100.0 * (frandom() - 0.5),
-        (float)mouseSpeedY / 2. - 120.0 * frandom(), (float)(1.5 + 0.5 * (frandom() - 0.5)),
-        (float)(11 + 3. * (frandom() - 0.5)), 0.8 + 0.2 * frandom(), 0.8 + 0.2 * frandom(),
-        0.5 + 0.2 * frandom(), 0.9 - 0.35 * frandom());
+    float pos[2] = {(float)(mouseX + 10.0f * (frandom() - 0.5f)),
+                    (float)(mouseY + 10.0f * (frandom() - 0.5f))};
+    float speed[2] = {(float)(mouseSpeedX / 2.f + 100.0f * (frandom() - 0.5f)),
+                      (float)(mouseSpeedY / 2.f - 120.0f * frandom())};
+    float color[4] = {(float)(0.8f + 0.2f * frandom()), (float)(0.8f + 0.2f * frandom()),
+                      (float)(0.5f + 0.2f * frandom()), (float)(0.9f - 0.35f * frandom())};
+    sparkle2D->add(pos, speed, (float)(1.5 + 0.5 * (frandom() - 0.5)),
+                   (float)(11 + 3. * (frandom() - 0.5)), color);
   }
 }
 
@@ -291,30 +285,28 @@ void draw2DQuad(const GLfloat ver[4][2], const GLfloat txc[4][2], const GLfloat
 
   static GLuint idxs = (GLuint)-1;
   static GLuint buf = (GLuint)-1;
+  static GLuint vao = (GLuint)-1;
   if (idxs == (GLuint)-1) {
     glGenBuffers(1, &idxs);
-    ushort idxdata[6] = {0, 1, 2, 1, 2, 3};
+    glGenBuffers(1, &buf);
+    glGenVertexArrays(1, &vao);
+
+    const ushort idxdata[6] = {0, 1, 2, 1, 2, 3};
+
+    glBindVertexArray(vao);
+    glBindBuffer(GL_ARRAY_BUFFER, buf);
+    glBufferData(GL_ARRAY_BUFFER, 32 * sizeof(GLfloat), NULL, GL_STREAM_DRAW);
     glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxs);
     glBufferData(GL_ELEMENT_ARRAY_BUFFER, 6 * sizeof(ushort), idxdata, GL_STATIC_DRAW);
-
-    glGenBuffers(1, &buf);
-    glBindBuffer(GL_ARRAY_BUFFER, buf);
-    glBufferData(GL_ARRAY_BUFFER, 32 * sizeof(GLfloat), data, GL_DYNAMIC_DRAW);
+    configureUIAttributes();
   } else {
-    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, idxs);
-    glBindBuffer(GL_ARRAY_BUFFER, buf);
-    glBufferSubData(GL_ARRAY_BUFFER, 0, 32 * sizeof(GLfloat), data);
+    glBindVertexArray(vao);
   }
 
+  glBindBuffer(GL_ARRAY_BUFFER, buf);
+  glBufferSubData(GL_ARRAY_BUFFER, 0, 32 * sizeof(GLfloat), data);
+
   glBindTexture(GL_TEXTURE_2D, tex);
-
-  // Input structure: 2x Position; 4x color; 2x texture coord
-  glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 8 * sizeof(GLfloat), (void *)0);
-  glVertexAttribPointer(1, 4, GL_FLOAT, GL_FALSE, 8 * sizeof(GLfloat),
-                        (void *)(2 * sizeof(GLfloat)));
-  glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, 8 * sizeof(GLfloat),
-                        (void *)(6 * sizeof(GLfloat)));
-
   glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, (void *)0);
 }
 
@@ -326,8 +318,8 @@ void drawMousePointer() {
 }
 
 void drawMouse(int x, int y, int w, int h) {
-  GLfloat r1 = 1.0 + 0.1 * cos(mousePointerPhase * 1.8);
-  GLfloat r2 = 1.0 + 0.1 * cos(mousePointerPhase * 1.9);
+  GLfloat r1 = 1.0 + 0.1 * std::cos(mousePointerPhase * 1.8);
+  GLfloat r2 = 1.0 + 0.1 * std::cos(mousePointerPhase * 1.9);
   GLfloat dx = 0.707f * w * r1 * std::sin(mousePointerPhase * 0.35);
   GLfloat dy = 0.707f * h * r2 * std::cos(mousePointerPhase * 0.35);
 
@@ -365,6 +357,17 @@ void configureObjectAttributes() {
   glEnableVertexAttribArray(2);
   glEnableVertexAttribArray(3);
 }
+void configureUIAttributes() {
+  // To be used by `shaderUI`
+  glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 8 * sizeof(GLfloat), (void *)0);
+  glVertexAttribPointer(1, 4, GL_FLOAT, GL_FALSE, 8 * sizeof(GLfloat),
+                        (void *)(2 * sizeof(GLfloat)));
+  glVertexAttribPointer(2, 2, GL_FLOAT, GL_FALSE, 8 * sizeof(GLfloat),
+                        (void *)(6 * sizeof(GLfloat)));
+  glEnableVertexAttribArray(0);
+  glEnableVertexAttribArray(1);
+  glEnableVertexAttribArray(2);
+}
 
 static GLuint lastProgram = 0;
 
@@ -441,7 +444,7 @@ void setActiveProgramAndUniforms(Shader_Type type) {
   case Shader_Object:
     uniformLocations.render_stage = glGetUniformLocation(shader, "render_stage");
     uniformLocations.specular_color = glGetUniformLocation(shader, "specular");
-    uniformLocations.shininess = glGetUniformLocation(shader, "shininess");
+    uniformLocations.sharpness = glGetUniformLocation(shader, "sharpness");
     uniformLocations.use_lighting = glGetUniformLocation(shader, "use_lighting");
     uniformLocations.ignore_shadow = glGetUniformLocation(shader, "ignore_shadow");
     break;
@@ -515,10 +518,10 @@ void setActiveProgramAndUniforms(Shader_Type type) {
     if (type != Shader_Line) { glActiveTexture(GL_TEXTURE0 + 0); }
   }
 }
-void setObjectUniforms(Color c, float shininess, Object_Lighting lighting) {
+void setObjectUniforms(Color c, float sharpness, Object_Lighting lighting) {
   if (lastProgram == shaderObjectDay || lastProgram == shaderObjectNight) {
     glUniformC(uniformLocations.specular_color, c);
-    glUniform1f(uniformLocations.shininess, shininess);
+    glUniform1f(uniformLocations.sharpness, sharpness);
     glUniform1i(uniformLocations.ignore_shadow, lighting == Lighting_Regular);
     glUniform1i(uniformLocations.use_lighting, lighting != Lighting_None);
   }
@@ -755,7 +758,7 @@ void renderShadowMap(const Coord3d &focus, Map *mp, Game *gm) {
                  upv[loop][2], activeView.modelview);
     markViewChanged();
     // Render (todo: 50% alpha clip)
-    if (mp) mp->draw(0, focus[0], focus[1]);
+    if (mp) mp->draw(0, focus, gm ? gm->gameTime : 0.);
     if (gm) gm->draw();
   }
 
@@ -777,7 +780,7 @@ void renderShadowCascade(const Coord3d &focus, Map *mp, Game *gm) {
   const int N = 3;
 
   GLfloat proj_half_angle = 40 / 2 * M_PI / 180;
-  GLfloat aspect = (GLdouble)screenWidth / (GLdouble)fmax(screenHeight, 1);
+  GLfloat aspect = (GLdouble)screenWidth / (GLdouble)std::max(screenHeight, 1);
 
   /* shadow step scale by factor of 4 */
   GLfloat dts[N + 1] = {0.f, 12.5f, 50.f, 200.f};
@@ -887,7 +890,7 @@ void renderShadowCascade(const Coord3d &focus, Map *mp, Game *gm) {
     assign(activeView.cascade_model[loop], activeView.modelview);
     assign(activeView.cascade_proj[loop], activeView.projection);
     markViewChanged();
-    if (mp) mp->draw(0, focus[0], focus[1]);
+    if (mp) mp->draw(0, focus, gm ? gm->gameTime : 0.);
     if (gm) gm->draw();
   }
 
@@ -966,8 +969,8 @@ int createSnapshot() {
 void message(char *A, char *B) {
   int size = 16;
   int w1, w2, h1, h2, w;
-  w1 = Font::getTextWidth(A, size);
-  w2 = Font::getTextWidth(B, size);
+  w1 = getTextWidth(A, size);
+  w2 = getTextWidth(B, size);
   h1 = 32;
   h2 = 32;
 
@@ -978,10 +981,8 @@ void message(char *A, char *B) {
   Enter2DMode();
   draw2DRectangle(x1, y1, x2 - x1, y2 - y1, 0., 0., 1., 1., 0.2, 0.5, 0.2, 0.5);
 
-  Font::drawCenterSimpleText(A, screenWidth / 2, screenHeight / 2 - size, size, 0.5, 1.0, 0.2,
-                             1.0);
-  Font::drawCenterSimpleText(B, screenWidth / 2, screenHeight / 2 + 14, size, 0.5, 1.0, 0.2,
-                             1.0);
+  drawCenterSimpleText(A, screenWidth / 2, screenHeight / 2 - size, size, 0.5, 1.0, 0.2, 1.0);
+  drawCenterSimpleText(B, screenWidth / 2, screenHeight / 2 + 14, size, 0.5, 1.0, 0.2, 1.0);
 
   Leave2DMode();
 }
@@ -1005,14 +1006,13 @@ void multiMessage(int nlines, const char *left[], const char *right[]) {
   for (int i = 0; i < nlines; i++) {
     h_now += 2 * size;
     if (left[i]) {
-      Font::drawSimpleText(left[i], screenWidth / 2 - width / 2,
-                           screenHeight / 2 - total_height / 2 + h_now, size, 0.5, 1.0, 0.2,
-                           1.0);
+      drawSimpleText(left[i], screenWidth / 2 - width / 2,
+                     screenHeight / 2 - total_height / 2 + h_now, size, 0.5, 1.0, 0.2, 1.0);
     }
     if (right[i]) {
-      Font::drawRightSimpleText(right[i], screenWidth / 2 + width / 2,
-                                screenHeight / 2 - total_height / 2 + h_now, size, 0.5, 1.0,
-                                0.2, 1.0);
+      drawRightSimpleText(right[i], screenWidth / 2 + width / 2,
+                          screenHeight / 2 - total_height / 2 + h_now, size, 0.5, 1.0, 0.2,
+                          1.0);
     }
   }
   Leave2DMode();
@@ -1052,20 +1052,88 @@ int loadTexture(const char *name) {
   return (numTextures - 1);  // ok
 }
 
+char *filetobuf(const char *filename) {
+  FILE *fptr = fopen(filename, "rb");
+  if (!fptr) return NULL;
+  fseek(fptr, 0, SEEK_END);
+  long length = ftell(fptr);
+  char *buf = (char *)malloc(length + 1);
+  fseek(fptr, 0, SEEK_SET);
+  fread(buf, length, 1, fptr);
+  fclose(fptr);
+  buf[length] = 0;
+  return buf;
+}
+
+static GLuint loadShaderPart(const char *name, GLuint shader_type) {
+  char path[256];
+  const char *tdesc = NULL;
+  if (shader_type == GL_VERTEX_SHADER) {
+    tdesc = "Vertex";
+  } else if (shader_type == GL_FRAGMENT_SHADER) {
+    tdesc = "Fragment";
+  }
+
+  snprintf(path, 256, "%s/shaders/%s", effectiveShareDir, name);
+  GLchar *source = filetobuf(path);
+  if (source == NULL) { error("%s shader %s could not be read", tdesc, path); }
+  GLuint shader = glCreateShader(shader_type);
+  if (shader == 0) { error("Failed to even create shader object"); }
+  glShaderSource(shader, 1, (const GLchar **)&source, 0);
+  free(source);
+
+  glCompileShader(shader);
+  int isCompiled = 0;
+  glGetShaderiv(shader, GL_COMPILE_STATUS, &isCompiled);
+  if (isCompiled == 0) {
+    int maxLength = 0;
+    glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &maxLength);
+    char *vertexInfoLog = (char *)malloc(maxLength);
+    glGetShaderInfoLog(shader, maxLength, &maxLength, vertexInfoLog);
+    warning("%s shader %s error (len %d): %s", tdesc, name, maxLength, vertexInfoLog);
+    glDeleteShader(shader);
+    free(vertexInfoLog);
+    return 0;
+  }
+  return shader;
+}
+
+static GLuint linkShader(GLuint vertexshader, GLuint fragmentshader, const char *descr) {
+  GLuint shaderprogram = glCreateProgram();
+  glAttachShader(shaderprogram, vertexshader);
+  glAttachShader(shaderprogram, fragmentshader);
+  /* Shaders use attribs 1..k for some k */
+  glBindAttribLocation(shaderprogram, 0, "in_Position");
+  glBindAttribLocation(shaderprogram, 1, "in_Color");
+  glBindAttribLocation(shaderprogram, 2, "in_Texcoord");
+  glBindAttribLocation(shaderprogram, 3, "in_Normal");
+  glBindAttribLocation(shaderprogram, 4, "in_Velocity");
+  glLinkProgram(shaderprogram);
+  int is_linked;
+  glGetProgramiv(shaderprogram, GL_LINK_STATUS, (int *)&is_linked);
+  glDeleteShader(vertexshader);
+  glDeleteShader(fragmentshader);
+  if (is_linked == 0) {
+    int maxLength = 0;
+    glGetProgramiv(shaderprogram, GL_INFO_LOG_LENGTH, &maxLength);
+    char *shaderProgramInfoLog = (char *)malloc(maxLength);
+    glGetProgramInfoLog(shaderprogram, maxLength, &maxLength, shaderProgramInfoLog);
+    warning("Program (%s) link error (len %d): %s", descr, maxLength, shaderProgramInfoLog);
+    free(shaderProgramInfoLog);
+    return 0;
+  }
+  return shaderprogram;
+}
+
 void glHelpInit() {
   warnForGLerrors("preGLinit");
 
-  for (int i = 0; i < 4711; i++) fake_rand[i] = frand();
-
   TTF_Init();
   char str[256];
   snprintf(str, sizeof(str), "%s/fonts/%s", effectiveShareDir, "menuFont.ttf");
   ingameFont = TTF_OpenFont(str, 30);
   if (!ingameFont) { error("failed to load font %s", str); }
 
-  // The VAO need only be there :-)
-  glGenVertexArrays(1, &defaultVao);
-
   /* Note: all textures must be powers of 2 since we ignore texcoords */
   loadTexture("ice.png");
   loadTexture("acid.png");
@@ -1083,18 +1151,61 @@ void glHelpInit() {
 
   sparkle2D = new Sparkle2D();
 
-  // Errors handled within loadProgram
-  shaderTileDay = loadProgram("basic.vert", "basic_day.frag");
-  shaderTileNight = loadProgram("basic.vert", "basic_night.frag");
-  shaderTileShadow = loadProgram("basic.vert", "basic_shadow.frag");
-  shaderLine = loadProgram("line.vert", "line.frag");
-  shaderWaterDay = loadProgram("water.vert", "water_day.frag");
-  shaderWaterNight = loadProgram("water.vert", "water_night.frag");
-  shaderUI = loadProgram("ui.vert", "ui.frag");
-  shaderObjectDay = loadProgram("object.vert", "object_day.frag");
-  shaderObjectNight = loadProgram("object.vert", "object_night.frag");
-  shaderObjectShadow = loadProgram("object.vert", "object_shadow.frag");
-  shaderReflection = loadProgram("reflection.vert", "reflection.frag");
+  // Errors handled in loadShaderPart and linkShader
+  GLuint vBasic = loadShaderPart("basic.vert", GL_VERTEX_SHADER);
+  GLuint vLine = loadShaderPart("line.vert", GL_VERTEX_SHADER);
+  GLuint vWater = loadShaderPart("water.vert", GL_VERTEX_SHADER);
+  GLuint vUI = loadShaderPart("ui.vert", GL_VERTEX_SHADER);
+  GLuint vObject = loadShaderPart("object.vert", GL_VERTEX_SHADER);
+  GLuint vReflection = loadShaderPart("reflection.vert", GL_VERTEX_SHADER);
+
+  GLuint fBasicDay = loadShaderPart("basic_day.frag", GL_FRAGMENT_SHADER);
+  GLuint fBasicNight = loadShaderPart("basic_night.frag", GL_FRAGMENT_SHADER);
+  GLuint fBasicShadow = loadShaderPart("basic_shadow.frag", GL_FRAGMENT_SHADER);
+  GLuint fLine = loadShaderPart("line.frag", GL_FRAGMENT_SHADER);
+  GLuint fWaterDay = loadShaderPart("water_day.frag", GL_FRAGMENT_SHADER);
+  GLuint fWaterNight = loadShaderPart("water_night.frag", GL_FRAGMENT_SHADER);
+  GLuint fUI = loadShaderPart("ui.frag", GL_FRAGMENT_SHADER);
+  GLuint fObjectDay = loadShaderPart("object_day.frag", GL_FRAGMENT_SHADER);
+  GLuint fObjectNight = loadShaderPart("object_night.frag", GL_FRAGMENT_SHADER);
+  GLuint fObjectShadow = loadShaderPart("object_shadow.frag", GL_FRAGMENT_SHADER);
+  GLuint fReflection = loadShaderPart("reflection.frag", GL_FRAGMENT_SHADER);
+
+  shaderTileDay = linkShader(vBasic, fBasicDay, "BasicDay");
+  shaderTileNight = linkShader(vBasic, fBasicNight, "BasicNight");
+  shaderTileShadow = linkShader(vBasic, fBasicShadow, "BasicShadow");
+  shaderLine = linkShader(vLine, fLine, "Line");
+  shaderWaterDay = linkShader(vWater, fWaterDay, "WaterDay");
+  shaderWaterNight = linkShader(vWater, fWaterNight, "WaterNight");
+  shaderUI = linkShader(vUI, fUI, "UI");
+  shaderObjectDay = linkShader(vObject, fObjectDay, "ObjectDay");
+  shaderObjectNight = linkShader(vObject, fObjectNight, "ObjectNight");
+  shaderObjectShadow = linkShader(vObject, fObjectShadow, "ObjectShadow");
+  shaderReflection = linkShader(vReflection, fReflection, "Reflection");
+
+  glDeleteShader(vBasic);
+  glDeleteShader(vLine);
+  glDeleteShader(vWater);
+  glDeleteShader(vUI);
+  glDeleteShader(vObject);
+  glDeleteShader(vReflection);
+
+  glDeleteShader(fBasicDay);
+  glDeleteShader(fBasicNight);
+  glDeleteShader(fBasicShadow);
+  glDeleteShader(fLine);
+  glDeleteShader(fWaterDay);
+  glDeleteShader(fWaterNight);
+  glDeleteShader(fUI);
+  glDeleteShader(fObjectDay);
+  glDeleteShader(fObjectNight);
+  glDeleteShader(fObjectShadow);
+  glDeleteShader(fReflection);
+
+  // Load uniform locations
+  uniformLocations.ui_screen_width = glGetUniformLocation(shaderUI, "screen_width");
+  uniformLocations.ui_screen_height = glGetUniformLocation(shaderUI, "screen_height");
+  uniformLocations.ui_tex = glGetUniformLocation(shaderUI, "tex");
 
   // Wipe ball cache
   for (int i = 0; i < MAX_BALL_DETAIL; i++) {
@@ -1128,6 +1239,7 @@ void glHelpInit() {
   activeView.sun_direction[2] = -93 / 121.;
 
   activeView.calculating_shadows = false;
+  activeView.show_flag_state = false;
 
   glEnable(GL_TEXTURE_CUBE_MAP_SEAMLESS);
   glGenTextures(1, &activeView.shadowMapTexture);
@@ -1174,7 +1286,6 @@ void glHelpCleanup() {
   if (shaderObjectNight) glDeleteProgram(shaderObjectNight);
   if (shaderObjectShadow) glDeleteProgram(shaderObjectShadow);
   if (shaderReflection) glDeleteProgram(shaderReflection);
-  if (defaultVao) glDeleteVertexArrays(1, &defaultVao);
   shaderLine = 0;
   shaderTileDay = 0;
   shaderTileNight = 0;
@@ -1185,7 +1296,6 @@ void glHelpCleanup() {
   shaderObjectDay = 0;
   shaderObjectNight = 0;
   shaderObjectShadow = 0;
-  defaultVao = 0;
 
   if (sparkle2D) delete sparkle2D;
 
@@ -1196,8 +1306,7 @@ void glHelpCleanup() {
   }
 
   /* Invalidate all strings so they get cleaned up */
-  stringTick += 100000;
-  update2DStringCache();
+  update2DStringCache(true);
 
   for (std::map<int, TTF_Font *>::iterator i = menuFontLookup.begin();
        i != menuFontLookup.end(); ++i) {
@@ -1210,97 +1319,28 @@ void glHelpCleanup() {
   for (int i = 0; i < numTextures; i++) { glDeleteTextures(1, &textures[i]); }
 }
 
-char *filetobuf(const char *filename) {
-  FILE *fptr = fopen(filename, "rb");
-  if (!fptr) return NULL;
-  fseek(fptr, 0, SEEK_END);
-  long length = ftell(fptr);
-  char *buf = (char *)malloc(length + 1);
-  fseek(fptr, 0, SEEK_SET);
-  fread(buf, length, 1, fptr);
-  fclose(fptr);
-  buf[length] = 0;
-  return buf;
-}
-
-GLuint loadProgram(const char *vertname, const char *fragname) {
-  /* Read our shaders into the appropriate buffers */
-  char path[256];
-  snprintf(path, 256, "%s/shaders/%s", effectiveShareDir, vertname);
-  GLchar *vertexsource = filetobuf(path);
-  if (vertexsource == NULL) { error("Vertex shader %s could not be read", path); }
-  snprintf(path, 256, "%s/shaders/%s", effectiveShareDir, fragname);
-  GLchar *fragmentsource = filetobuf(path);
-  if (fragmentsource == NULL) { error("Fragment shader %s could not be read", path); }
-  GLuint vertexshader = glCreateShader(GL_VERTEX_SHADER);
-  int maxLength;
-  glShaderSource(vertexshader, 1, (const GLchar **)&vertexsource, 0);
-  glCompileShader(vertexshader);
-  int IsCompiled_VS, IsCompiled_FS, IsLinked;
-  glGetShaderiv(vertexshader, GL_COMPILE_STATUS, &IsCompiled_VS);
-  if (IsCompiled_VS == 0) {
-    glGetShaderiv(vertexshader, GL_INFO_LOG_LENGTH, &maxLength);
-    char *vertexInfoLog = (char *)malloc(maxLength);
-    glGetShaderInfoLog(vertexshader, maxLength, &maxLength, vertexInfoLog);
-    warning("Vertex shader %s error: %s", vertname, vertexInfoLog);
-    glDeleteShader(vertexshader);
-    free(vertexInfoLog);
-    free(vertexsource);
-    free(fragmentsource);
-    return -1;
-  }
-  GLuint fragmentshader = glCreateShader(GL_FRAGMENT_SHADER);
-  glShaderSource(fragmentshader, 1, (const GLchar **)&fragmentsource, 0);
-  glCompileShader(fragmentshader);
-  glGetShaderiv(fragmentshader, GL_COMPILE_STATUS, &IsCompiled_FS);
-  if (IsCompiled_FS == 0) {
-    glGetShaderiv(fragmentshader, GL_INFO_LOG_LENGTH, &maxLength);
-    char *fragmentInfoLog = (char *)malloc(maxLength);
-    glGetShaderInfoLog(fragmentshader, maxLength, &maxLength, fragmentInfoLog);
-    warning("Fragment shader %s error: %s", fragname, fragmentInfoLog);
-    glDeleteShader(vertexshader);
-    glDeleteShader(fragmentshader);
-    free(fragmentInfoLog);
-    free(vertexsource);
-    free(fragmentsource);
-    return -1;
-  }
-  free(vertexsource);
-  free(fragmentsource);
-  GLuint shaderprogram = glCreateProgram();
-  glAttachShader(shaderprogram, vertexshader);
-  glAttachShader(shaderprogram, fragmentshader);
-  /* Shaders use attribs 1..k for some k */
-  glBindAttribLocation(shaderprogram, 0, "in_Position");
-  glBindAttribLocation(shaderprogram, 1, "in_Color");
-  glBindAttribLocation(shaderprogram, 2, "in_Texcoord");
-  glBindAttribLocation(shaderprogram, 3, "in_Normal");
-  glBindAttribLocation(shaderprogram, 4, "in_Velocity");
-  glLinkProgram(shaderprogram);
-  glGetProgramiv(shaderprogram, GL_LINK_STATUS, (int *)&IsLinked);
-  glDeleteShader(vertexshader);
-  glDeleteShader(fragmentshader);
-  if (IsLinked == 0) {
-    glGetProgramiv(shaderprogram, GL_INFO_LOG_LENGTH, &maxLength);
-    char *shaderProgramInfoLog = (char *)malloc(maxLength);
-    glGetProgramInfoLog(shaderprogram, maxLength, &maxLength, shaderProgramInfoLog);
-    warning("Program (%s %s) link error: %s", vertname, fragname, shaderProgramInfoLog);
-    free(shaderProgramInfoLog);
-    return -1;
-  }
-
-  return shaderprogram;
-}
-
 void warnForGLerrors(const char *where_am_i) {
   GLenum err;
   while ((err = glGetError()) != GL_NO_ERROR) {
-    warning("GL error %x at location: %s", err, where_am_i);
+    const char *type = NULL;
+    switch (err) {
+    default:
+      break;
+    case GL_INVALID_ENUM:
+      type = "Invalid enum";
+      break;
+    case GL_INVALID_VALUE:
+      type = "Invalid value";
+      break;
+    case GL_INVALID_OPERATION:
+      type = "Invalid operation";
+      break;
+    }
+
+    warning("GL error %x (%s) at location: %s", err, type, where_am_i);
   }
 }
 
-#define FRAME 50
-
 /* Calculates and displays current framerate */
 void displayFrameRate() {
   /* Frame rate is given for the previous period. */
@@ -1424,7 +1464,7 @@ bool testBboxClip(double x1, double x2, double y1, double y2, double z1, double
   double vxh = -1e99, vyh = -1e99, vzh = -1e99;
   for (int i = 0; i < 8; i++) {
     double w[3] = {0., 0., 0.};
-    double p[3] = {i & 4 ? x1 : x2, i & 2 ? y1 : y2, i & 1 ? z1 : z2};
+    double p[3] = {(i & 4) ? x1 : x2, (i & 2) ? y1 : y2, (i & 1) ? z1 : z2};
     // Apply perspective transform with MVP matrix
     for (int k = 0; k < 3; k++)
       w[k] += mvp[0][k] * p[0] + mvp[1][k] * p[1] + mvp[2][k] * p[2] + mvp[3][k];
@@ -1446,10 +1486,10 @@ bool testBboxClip(double x1, double x2, double y1, double y2, double z1, double
   return true;
 }
 
-static int is_2d_mode = 0;
+static bool is_2d_mode = false;
 
 void Require2DMode() {
-  if (!is_2d_mode) { warning("Function should only be called in 2D mode"); }
+  if (!is_2d_mode) { warning("Function `Require2DMode` should only be called in 2D mode"); }
 }
 
 void Enter2DMode() {
@@ -1461,35 +1501,28 @@ void Enter2DMode() {
   glEnable(GL_BLEND);
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-  // Load program, bind VAO, and set up uniforms
+  // Load program and set up uniforms. VAO will by bound at point of use
   glUseProgram(shaderUI);
+  glUniform1f(uniformLocations.ui_screen_width, (GLfloat)screenWidth);
+  glUniform1f(uniformLocations.ui_screen_height, (GLfloat)screenHeight);
+  glUniform1i(uniformLocations.ui_tex, 0);
 
-  glBindVertexArray(defaultVao);
-
-  glUniform1f(glGetUniformLocation(shaderUI, "screen_width"), (GLfloat)screenWidth);
-  glUniform1f(glGetUniformLocation(shaderUI, "screen_height"), (GLfloat)screenHeight);
-
-  glEnableVertexAttribArray(0);
-  glEnableVertexAttribArray(1);
-  glEnableVertexAttribArray(2);
-
-  glUniform1i(glGetUniformLocation(shaderUI, "tex"), 0);
   glActiveTexture(GL_TEXTURE0);
-
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 
-  is_2d_mode = 1;
+  is_2d_mode = true;
 }
 
 void Leave2DMode() {
-  if (!is_2d_mode) { return; }
+  if (!is_2d_mode) {
+    warning("Function `Leave2DMode` should only be called in 2D mode");
+    return;
+  }
 
-  is_2d_mode = 0;
-
-  glBindVertexArray(0);
+  is_2d_mode = false;
 }
 
 int powerOfTwo(int input) {
