@@ -32,7 +32,7 @@ class HallOfFameMode : public GameMode {
   void activated();
   void deactivated();
   void key(int);
-  void idle(Real);
+  void tick(Real);
   void mouseDown(int state, int x, int y);
 
   static HallOfFameMode *init();
