@@ -24,6 +24,8 @@
 #include "gameMode.h"
 #include "glHelp.h"
 
+class Gamer;
+
 class MainMode : public GameMode {
  public:
   MainMode();
@@ -31,7 +33,7 @@ class MainMode : public GameMode {
 
   void display();
   void key(int);
-  void idle(Real td);
+  void tick(Real td);
   void doExpensiveComputations();
   void mouse(int state, int x, int y);
   void mouseDown(int button, int x, int y);
@@ -48,7 +50,7 @@ class MainMode : public GameMode {
   void levelComplete();
   void bonusLevelComplete();
 
-  static void setupLighting(bool isNight);
+  static void setupLighting(const Game *game, bool isNight);
   void renderEnvironmentTexture(GLuint texture, const Coord3d &focus) const;
   Real flash;
 
@@ -56,6 +58,8 @@ class MainMode : public GameMode {
   static void cleanup();
   static MainMode *mainMode;
 
+  Game *game;
+
   double zAngle, wantedZAngle, xyAngle, wantedXYAngle;
   Coord3d camFocus, camDelta;
   Matrix4d cameraModelView;
@@ -79,8 +83,6 @@ class MainMode : public GameMode {
     statusBonusLevelComplete,
     statusPaused
   } gameStatus;
-
-  char *mapname;
 };
 
 #endif
