@@ -272,7 +272,7 @@ static int testDir() {
   return 1;
 }
 
-static void *mainLoop(void* data) {
+static void *mainLoop(void *data) {
   /* OpenGL work is now *only* performed on this thread. */
   SDL_GLContext context = (SDL_GLContext)data;
   int r = SDL_GL_MakeCurrent(window, context);
@@ -505,14 +505,33 @@ static void *mainLoop(void* data) {
   return (void *)EXIT_SUCCESS;
 }
 
-static void *initSettings(void *) {
-  Settings::init();
-  return NULL;
+static bool setupLocaleAndTranslations() {
+  /* Initialialize i18n with system defaults, before anything else (eg. settings) have been
+   * loaded */
+  setlocale(LC_ALL, "");
+  char localedir[512];
+
+#ifdef LOCALEDIR
+  snprintf(localedir, 511, "%s", LOCALEDIR);
+#else
+  snprintf(localedir, 511, "%s/locale", effectiveShareDir);
+#endif
+  if (bindtextdomain(PACKAGE, localedir) == NULL) {
+    warning("Failed to set text domain directory to %s", localedir);
+    return false;
+  }
+  if (bind_textdomain_codeset(PACKAGE, "UTF-8") == NULL) {
+    warning("Failed to set text codeset to UTF-8");
+    return false;
+  }
+  if (textdomain(PACKAGE) == NULL) {
+    warning("Failed to set message domain");
+    return false;
+  }
+  return true;
 }
 
-int main(int argc, char **argv) {
-  const char *program_name = argv[0];
-
+static bool setupEnvAndPaths(const char *program_name) {
   /* Store the username, for use as a default player/level author name */
   const char *env_user = getenv("USER");
   if (env_user) {
@@ -586,15 +605,16 @@ int main(int argc, char **argv) {
     mkdir(effectiveLocalDir, S_IXUSR | S_IRUSR | S_IWUSR | S_IXGRP | S_IRGRP | S_IWGRP);
 
   if (NULL == getenv("GUILE_LOAD_PATH")) {
-    char guileLoadPath[256 + 16]; /*longest effective share directory plus"GUILE_LOAD_PATH="*/
+    static char
+        guileLoadPath[256 + 16]; /*longest effective share directory plus"GUILE_LOAD_PATH="*/
     snprintf(guileLoadPath, sizeof(guileLoadPath), "GUILE_LOAD_PATH=%s", effectiveShareDir);
     putenv(guileLoadPath);
   }
 
 #ifdef WIN32
   if (NULL == getenv("HOME")) {
+    static char homeEnv[MAX_PATH + 5];
     char homePath[MAX_PATH];
-    char homeEnv[MAX_PATH + 5];
     SHGetFolderPath(NULL, CSIDL_APPDATA, NULL, 0, homePath);
     snprintf(homeEnv, sizeof(homeEnv), "HOME=%s", homePath);
     for (int i = strlen(homeEnv) - 1; i >= 0; i--)
@@ -603,24 +623,18 @@ int main(int argc, char **argv) {
     putenv(homeEnv);
   }
 #endif
+  return true;
+}
 
-  /* Initialialize i18n with system defaults, before anything else (eg. settings) have been
-   * loaded */
-  setlocale(LC_ALL, "");
-  char localedir[512];
+int main(int argc, char **argv) {
+  const char *program_name = argv[0];
 
-#ifdef LOCALEDIR
-  snprintf(localedir, 511, "%s", LOCALEDIR);
-#else
-  snprintf(localedir, 511, "%s/locale", effectiveShareDir);
-#endif
-  bindtextdomain(PACKAGE, localedir);
-  textdomain(PACKAGE);
+  if (!setupEnvAndPaths(program_name)) { return EXIT_FAILURE; }
 
-  /* Loading the settings uses Guile for parsing, but we don't need it again
-   * until e.g. loading the highscores/gamer info, which is done in the non-input
-   * thread */
-  scm_with_guile(initSettings, NULL);
+  if (!setupLocaleAndTranslations()) { return EXIT_FAILURE; }
+
+  /* This calls scm_with_guile which initializes guile which requires env setup */
+  Settings::init();
 
   startInEditMode = false;
   bool touchMode = false;
@@ -644,8 +658,6 @@ int main(int argc, char **argv) {
                                         {NULL, 0, NULL, 0}};
   int next_option;
 
-  program_name = argv[0];
-
   displayStartTime = getMonotonicTime();
   lastDisplayStartTime = displayStartTime;
   lastDisplayStartTime.tv_sec -= 1;
@@ -760,7 +772,7 @@ int main(int argc, char **argv) {
     warning("capslock is on, the mouse will be visible and not grabbed");
   }
 
-  void *v = scm_with_guile(mainLoop, gl_context);
+  void *v = mainLoop(gl_context);
   int retval = (int)(ptrdiff_t)v;
 
   SDL_Quit();
